#include "HX711.h"
#include <Wire.h>
#include <DS3231.h>
#include <Servo.h>

const bool SET_DATE = false;

const int LOADCELL_DOUT_PIN = 2;
const int LOADCELL_SCK_PIN = 3;

const int STROBE = 4;
const int DATA = 5;
const int CLK = 6;
const int OE = 7;

const int BUTTON1 = 8;
const int BUTTON2 = 10;
const int BUTTON3 = 11;
const int BUTTON4 = 12;
const int BUTTON5 = 13;

const int SERVO_PWM = 9;

const long LOADCELL_OFFSET = 106225;
const long LOADCELL_DIVIDER = 525;

const int digit_encoding[11] = {192, 249, 164, 176, 153, 146, 130, 248, 128, 144, 255};
const int pow10[4] = {1, 10, 100, 1000};

HX711 loadcell;
DS3231 RTC;
Servo servo;

byte RTC_second, RTC_minute, RTC_hour, RTC_dow, RTC_date, RTC_month, RTC_year;
bool RTC_hour12, RTC_am_pm, RTC_century;

int dot_points = 0b0100, number_digit = 0, show_digit = 0b1111, blinkMask = 0b1111, scrollDigit;
int num, date_num, special_num;

bool set_alarm_flag, set_portion_flag, is_feeding_flag;
bool lastButtonState[6], buttonState[6];
unsigned long lastDebounceTime[6], lastBlinkTime, lastFeedTime, init_weight;  // the last time the output pin was toggled
unsigned long debounceDelay = 50, blinkDelay = 500, feedDelay = 65000;
int alarms[10], alarms_count = 0, portion_size;


void read_date() {
  RTC_second = RTC.getSecond();
  RTC_minute = RTC.getMinute();
  RTC_hour = RTC.getHour(RTC_hour12, RTC_am_pm);
  RTC_dow = RTC.getDoW();
  RTC_date = RTC.getDate();
  RTC_month = RTC.getMonth(RTC_century);
  RTC_year = RTC.getYear();
}

void set_date(byte second, byte minute, byte hour, byte date, byte month, byte year) {
  RTC.setSecond(second);
  RTC.setMinute(minute);
  RTC.setHour(hour);
  RTC.setDate(date);
  RTC.setMonth(month);
  RTC.setYear(year);
}

void write_byte(byte data) {
  digitalWrite(STROBE, LOW);
  shiftOut(DATA, CLK, LSBFIRST, data);
  digitalWrite(STROBE, HIGH);
}

void turn_on_digit(byte digit, byte pos, byte dot_point) {
  digitalWrite(OE, LOW);
  write_byte(15 - (1 << pos));
  write_byte(digit_encoding[digit] ^ (dot_point << 7));
  digitalWrite(OE, HIGH);
}

void draw_number() {
  int dig;

  if (set_alarm_flag || set_portion_flag) {
    num = special_num;
  } else {
    num = date_num;
  }
  
  if(((/*show_digit*/blinkMask >> number_digit) & 1) == 0) {
    dig = 10;
  } else {
    dig = num / pow10[number_digit] % 10;
  }
  
  turn_on_digit(dig, 3 - number_digit, (dot_points >> number_digit) & 1);

  number_digit++;
  if(number_digit == 4) {
    number_digit = 0;
  }
  delay(1);
}


void check_button_up() {
  bool reading = digitalRead(BUTTON5);

  if (reading != lastButtonState[5]) {
    lastDebounceTime[5] = millis();
  }
  
  if ((millis() - lastDebounceTime[5]) > debounceDelay) {
    if (reading != buttonState[5]) {
      buttonState[5] = reading;

      if (buttonState[5] == HIGH && (set_alarm_flag || set_portion_flag)) {
        Serial.println("amogus");
        int cif = special_num / pow10[scrollDigit] % 10;
        if (cif == 9) {
          special_num -= 9 * pow10[scrollDigit];
        } else {
          special_num += pow10[scrollDigit];
        }
        lastBlinkTime = millis();
        blinkMask = 0b1111;
      }
    }
  }

  lastButtonState[5] = reading;
}

void check_button_down() {
  bool reading = digitalRead(BUTTON4);

  if (reading != lastButtonState[4]) {
    lastDebounceTime[4] = millis();
  }
  
  if ((millis() - lastDebounceTime[4]) > debounceDelay) {
    if (reading != buttonState[4]) {
      buttonState[4] = reading;

      if (buttonState[4] == HIGH && (set_alarm_flag || set_portion_flag)) {
        Serial.println("amogus");
        int cif = special_num / pow10[scrollDigit] % 10;
        if (cif == 0) {
          special_num += 9 * pow10[scrollDigit];
        } else {
          special_num -= pow10[scrollDigit];
        }
        lastBlinkTime = millis();
        blinkMask = 0b1111;
      }
    }
  }

  lastButtonState[4] = reading;
}

void check_button_scroll() {
  bool reading = digitalRead(BUTTON3);

  if (reading != lastButtonState[3]) {
    lastDebounceTime[3] = millis();
  }
  
  if ((millis() - lastDebounceTime[3]) > debounceDelay) {
    if (reading != buttonState[3]) {
      buttonState[3] = reading;

      if (buttonState[3] == HIGH) {
        blinkMask = 0b1111;
        scrollDigit = (scrollDigit - 1);
        if (scrollDigit == -1) {
          scrollDigit = 3;
        }

        lastBlinkTime = millis() - blinkDelay;
      }
    }
  }

  lastButtonState[3] = reading;
}

void setup_alarm_num() {
  scrollDigit = 3;
  special_num = 2100;
}

void setup_portion_num() {
  scrollDigit = 3;
  special_num = 100;
}


void check_button_alarm() {
  bool reading = digitalRead(BUTTON2);

  if (reading != lastButtonState[2]) {
    lastDebounceTime[2] = millis();
  }
  
  if ((millis() - lastDebounceTime[2]) > debounceDelay) {
    if (reading != buttonState[2]) {
      buttonState[2] = reading;

      if (buttonState[2] == HIGH) {
        if (set_portion_flag == 1) {
          set_portion_flag = 0;
          dot_points = 0b0100;
        } else if (set_alarm_flag == 0) {
          set_alarm_flag = 1;
          setup_alarm_num();
        } else {
          set_alarm_flag = 0;
          alarms[alarms_count++] = special_num;
          Serial.println(special_num);
        }
      }
    }
  }

  lastButtonState[2] = reading;
}

void check_button_portion() {
  bool reading = digitalRead(BUTTON1);

  if (reading != lastButtonState[1]) {
    lastDebounceTime[1] = millis();
  }
  
  if ((millis() - lastDebounceTime[1]) > debounceDelay) {
    if (reading != buttonState[1]) {
      buttonState[1] = reading;

      if (buttonState[1] == HIGH) {
        if (set_alarm_flag == 1) {
          set_alarm_flag = 0;
        } else if (set_portion_flag == 0) {
          set_portion_flag = 1;
          setup_portion_num();
          dot_points = 0b0000;
        } else {
          set_portion_flag = 0;
          dot_points = 0b0100;
          portion_size = special_num;
          Serial.println(special_num);
        }
      }
    }
  }

  lastButtonState[1] = reading;
}

void setup() {
  Serial.begin(9600);

  servo.attach(SERVO_PWM);
  servo.write(0);

  Wire.begin();
  RTC.setClockMode(false);

  if(SET_DATE) {
    set_date(0, 53, 17, 1, 6, 22);
  }

  // 3. Initialize library
  loadcell.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);
  loadcell.set_scale(LOADCELL_DIVIDER);
  loadcell.set_offset(LOADCELL_OFFSET);

  pinMode(STROBE, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(CLK, OUTPUT);
  pinMode(OE, OUTPUT);

  pinMode(BUTTON1, INPUT);
  pinMode(BUTTON2, INPUT);
  pinMode(BUTTON3, INPUT);
  pinMode(BUTTON4, INPUT);
  pinMode(BUTTON5, INPUT_PULLUP);
  
  digitalWrite(OE, HIGH);

  set_alarm_flag = set_portion_flag = 0;
  is_feeding_flag = 0;

  portion_size = 20;
}

void loop() { 
  read_date();
  date_num = RTC_hour * 100 + RTC_minute;

  check_button_alarm();
  check_button_portion();

  check_button_scroll();
  check_button_up();
  check_button_down();

  if (set_alarm_flag || set_portion_flag) {
    if ((millis() - lastBlinkTime) > blinkDelay) {
      lastBlinkTime = millis();
      blinkMask ^= (1 << scrollDigit);
      //Serial.println(blinkMask);
    }
  } else {
    blinkMask = 0b1111;
  }

  if ((millis() - lastFeedTime) > feedDelay && !is_feeding_flag) {
    for (int i = 0; i < alarms_count; ++i) {
      if (date_num == alarms[i]) {
        Serial.println("AMOGUSUS");
        servo.write(180);
        lastFeedTime = millis();
        is_feeding_flag = 1;
        init_weight = loadcell.get_units();
        break;
      }
    }
  }

  if (is_feeding_flag) {
    Serial.println(loadcell.get_units());
    if (init_weight - loadcell.get_units() > portion_size) {
      is_feeding_flag = 0;
      servo.write(0);
    }
  }

  draw_number();
}
